@@ -1,7 +1,7 @@
 /*
  *  Copyright © 1998, 2003 Jonathan Blandford <jrb@alum.mit.edu>
  *  Copyright © 2003 Callum McKenzie <callum@physics.otago.ac.nz>
- *  Copyright © 2007 Christian Persch
+ *  Copyright © 2007, 2008 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "config.h"
+#include <config.h>
 
 #include <string.h>
 #include <unistd.h>
@@ -49,7 +49,6 @@
 #endif
 #endif /* HAVE_HILDON */
 
-#include <libgames-support/games-card-common.h>
 #include <libgames-support/games-clock.h>
 #include <libgames-support/games-files.h>
 #include <libgames-support/games-stock.h>
@@ -268,7 +267,7 @@ select_game_cb (GtkAction *action,
                                         GTK_WINDOW (window),
                                         GTK_DIALOG_DESTROY_WITH_PARENT,
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                        GTK_STOCK_OK, GTK_RESPONSE_OK,
+                                        _("_Select"), GTK_RESPONSE_OK,
                                         NULL);
   gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
                                            GTK_RESPONSE_OK,
@@ -567,6 +566,7 @@ help_about_cb (GtkAction *action,
     "Rosanna Yuen <zana@webwynk.net>",
     "Alan Horkan <horkana@maths.tcd.ie>",
     "Richard Hoelscher <rah@rahga.com>",
+    "Vincent Povirk",
     NULL
   };
   const char *artists[] = {
@@ -613,7 +613,7 @@ help_about_cb (GtkAction *action,
 			     "AisleRiot is a part of GNOME Games."),
 #endif
                          "copyright", "Copyright © 1998-2006 Jonathan Blandford\n"
-                                      "Copyright © 2007 Christian Persch",
+                                      "Copyright © 2007, 2008 Christian Persch",
                          "license", licence,
                          "authors", authors,
                          "artists", artists,
@@ -735,31 +735,86 @@ debug_exception_cb (GtkAction *action,
   aisleriot_game_generate_exception (priv->game);
 }
 
+#define DEBUG_WINDOW_DATA_KEY "debug-data"
+
 typedef struct {
   AisleriotWindow *window;
   GList *games_list;
-} DebugCycleData;
+  GList *current_game;
+} DebugWindowData;
+
+static void
+debug_data_free (DebugWindowData *data)
+{
+  g_list_foreach (data->games_list, (GFunc) g_free, NULL);
+  g_list_free (data->games_list);
+  g_slice_free (DebugWindowData, data);
+}
+
+static DebugWindowData *
+debug_ensure_game_list (AisleriotWindow *window)
+{
+  AisleriotWindowPrivate *priv = window->priv;
+  DebugWindowData *data;
+  GamesFileList *files;
+  GList *l;
+
+  data = g_object_get_data (G_OBJECT (window), DEBUG_WINDOW_DATA_KEY);
+  if (data != NULL)
+    return data;
+
+  files = games_file_list_new ("*.scm", games_path_runtime_fix (GAMESDIR), NULL);
+  games_file_list_transform_basename (files);
+
+  data = g_slice_new (DebugWindowData);
+  data->window = window;
+  data->games_list = files->list;
+  data->current_game = g_list_find_custom (data->games_list,
+                                           aisleriot_game_get_game_file (priv->game),
+                                           (GCompareFunc) strcmp);
+
+  /* Remove the base sol.scm from the list */
+  for (l = data->games_list; l != NULL; l = l->next) {
+    char *game_file = (char *) l->data;
+
+    if (game_file != NULL &&
+        strcmp (game_file, "sol.scm") == 0) {
+      data->games_list = g_list_delete_link (data->games_list, l);
+      g_free (game_file);
+      break;
+    }
+  }
+
+  files->list = NULL;
+  g_object_unref (files);
+
+  g_object_set_data_full (G_OBJECT (window), DEBUG_WINDOW_DATA_KEY,
+                          data, (GDestroyNotify) debug_data_free);
+
+  return data;
+}
 
 static gboolean
-debug_cycle_timeout_cb (DebugCycleData *data)
+debug_cycle_timeout_cb (AisleriotWindow *window)
 {
+  DebugWindowData *data;
   char *game_file;
 
-  if (data->games_list == NULL) {
-    g_slice_free (DebugCycleData, data);
+  data = debug_ensure_game_list (window);
+  if (data->current_game != NULL) {
+    data->current_game = data->current_game->next;
+    /* We're done */
+    if (!data->current_game)
+      return FALSE;
+  }
+  if (!data->current_game) {
+    data->current_game = data->games_list;
+  }
+  if (!data->current_game)
     return FALSE;
-  }
 
-  game_file = data->games_list->data;
-  /* Take the head off of the list */
-  data->games_list = g_list_delete_link (data->games_list, data->games_list);
-
-  if (game_file != NULL &&
-      strcmp (game_file, "sol.scm") != 0) {
-    aisleriot_window_set_game (data->window, game_file, 0);
-  }
-
-  g_free (game_file);
+  game_file = data->current_game->data;  
+  aisleriot_window_set_game (data->window, game_file, 0);
 
   return TRUE;
 }
@@ -768,20 +823,73 @@ static void
 debug_cycle_cb (GtkAction *action,
                 AisleriotWindow *window)
 {
-  GamesFileList *files;
-  DebugCycleData *data;
+  g_timeout_add (500, (GSourceFunc) debug_cycle_timeout_cb, window);
+}
 
-  files = games_file_list_new ("*.scm", games_path_runtime_fix (GAMESDIR), NULL);
-  games_file_list_transform_basename (files);
+static void
+debug_game_first (GtkAction *action,
+                  AisleriotWindow *window)
+{
+  DebugWindowData *data;
 
-  data = g_slice_new (DebugCycleData);
-  data->window = window;
-  data->games_list = files->list;
+  data = debug_ensure_game_list (window);
+  data->current_game = data->games_list;
+  if (!data->current_game)
+    return;
 
-  files->list = NULL;
-  g_object_unref (files);
+  aisleriot_window_set_game (data->window, (const char *) data->current_game->data, 0);
+}
 
-  g_timeout_add (500, (GSourceFunc) debug_cycle_timeout_cb, data);
+static void
+debug_game_last (GtkAction *action,
+                 AisleriotWindow *window)
+{
+  DebugWindowData *data;
+
+  data = debug_ensure_game_list (window);
+  data->current_game = g_list_last (data->games_list);
+  if (!data->current_game)
+    return;
+
+  aisleriot_window_set_game (data->window, (const char *) data->current_game->data, 0);
+}
+
+static void
+debug_game_next (GtkAction *action,
+                 AisleriotWindow *window)
+{
+  DebugWindowData *data;
+
+  data = debug_ensure_game_list (window);
+  if (data->current_game) {
+    data->current_game = data->current_game->next;
+  }
+  if (!data->current_game) {
+    data->current_game = data->games_list;
+  }
+  if (!data->current_game)
+    return;
+
+  aisleriot_window_set_game (data->window, (const char *) data->current_game->data, 0);
+}
+
+static void
+debug_game_prev (GtkAction *action,
+                 AisleriotWindow *window)
+{
+  DebugWindowData *data;
+
+  data = debug_ensure_game_list (window);
+  if (data->current_game) {
+    data->current_game = data->current_game->prev;
+  }
+  if (!data->current_game) {
+    data->current_game = data->games_list;
+  }
+  if (!data->current_game)
+    return;
+
+  aisleriot_window_set_game (data->window, (const char *) data->current_game->data, 0);
 }
 
 #endif /* !HAVE_HILDON */
@@ -805,6 +913,8 @@ debug_choose_seed_response_cb (GtkWidget *dialog,
     seed = g_ascii_strtoull (text, &endptr, 10);
     if (errno == 0 && endptr != text) {
       aisleriot_game_new_game (priv->game, &seed);
+
+      gtk_widget_grab_focus (GTK_WIDGET (priv->board));
     }
   }
 
@@ -926,7 +1036,6 @@ static void
 sound_toggle_cb (GtkToggleAction *action,
                  AisleriotWindow *window)
 {
-  AisleriotWindowPrivate *priv = window->priv;
   gboolean sound_enabled;
 
   sound_enabled = gtk_toggle_action_get_active (action);
@@ -1948,9 +2057,10 @@ aisleriot_window_init (AisleriotWindow *window)
     /* Menu item actions */
     { "NewGame", GAMES_STOCK_NEW_GAME, NULL,
       ACTION_ACCEL ("<ctrl>N", NULL),
-      NULL,
+      ACTION_TOOLTIP (N_("Start a new game")),
       G_CALLBACK (new_game_cb) },
-    { "RestartGame", GAMES_STOCK_RESTART_GAME, NULL, NULL, NULL,
+    { "RestartGame", GAMES_STOCK_RESTART_GAME, NULL, NULL,
+       ACTION_TOOLTIP (N_("Restart the game")),
       G_CALLBACK (restart_game) },
     { "Select", GTK_STOCK_INDEX, N_("_Select Game..."),
       ACTION_ACCEL ("<ctrl>O", NULL),
@@ -1960,16 +2070,20 @@ aisleriot_window_init (AisleriotWindow *window)
     { "Statistics", GTK_STOCK_ADD, N_("S_tatistics"), NULL,
       ACTION_TOOLTIP (N_("Show gameplay statistics")),
       G_CALLBACK (statistics_cb) },
-    { "CloseWindow", GTK_STOCK_CLOSE, NULL, NULL, NULL,
+    { "CloseWindow", GTK_STOCK_CLOSE, NULL, NULL,
+      ACTION_TOOLTIP (N_("Close this window")),
       G_CALLBACK (close_window_cb) },
-    { "UndoMove", GAMES_STOCK_UNDO_MOVE, NULL, NULL, NULL,
+    { "UndoMove", GAMES_STOCK_UNDO_MOVE, NULL, NULL,
+      ACTION_TOOLTIP (N_("Undo the last move")),
       G_CALLBACK (undo_cb) },
-    { "RedoMove", GAMES_STOCK_REDO_MOVE, NULL, NULL, NULL,
+    { "RedoMove", GAMES_STOCK_REDO_MOVE, NULL, NULL,
+      ACTION_TOOLTIP (N_("Redo the undone move")),
       G_CALLBACK (redo_cb) },
-    { "Deal", GTK_STOCK_OK /* FIXMEchpe */, _("_Deal"), "<control>D",
+    { "Deal", GAMES_STOCK_DEAL_CARDS, NULL, NULL,
       ACTION_TOOLTIP (N_("Deal next card or cards")),
       G_CALLBACK (deal_cb) },
-    { "Hint", GAMES_STOCK_HINT, NULL, NULL, NULL,
+    { "Hint", GAMES_STOCK_HINT, NULL, NULL,
+      ACTION_TOOLTIP (N_("Get a hint for your next move")),
       G_CALLBACK (show_hint_cb) },
     { "Contents", GAMES_STOCK_CONTENTS, NULL, NULL,
       ACTION_TOOLTIP (N_("View help for Aisleriot")),
@@ -1978,7 +2092,8 @@ aisleriot_window_init (AisleriotWindow *window)
       ACTION_ACCEL ("<shift>F1", NULL),
       ACTION_TOOLTIP (N_("View help for this game")),
       G_CALLBACK (help_on_game_cb) },
-    { "About", GTK_STOCK_ABOUT, NULL, NULL, NULL,
+    { "About", GTK_STOCK_ABOUT, NULL, NULL,
+      ACTION_TOOLTIP (N_("About this game")),
       G_CALLBACK (help_about_cb) },
 
     /* Toolbar-only actions */
@@ -2004,6 +2119,14 @@ aisleriot_window_init (AisleriotWindow *window)
       G_CALLBACK (debug_exception_cb) },
     { "DebugCycle", NULL, "Cycle through _all games", NULL, NULL,
       G_CALLBACK (debug_cycle_cb) },
+    { "DebugGameFirst", GTK_STOCK_GOTO_FIRST, NULL, NULL, NULL,
+      G_CALLBACK (debug_game_first) },
+    { "DebugGameLast", GTK_STOCK_GOTO_LAST, NULL, NULL, NULL,
+      G_CALLBACK (debug_game_last) },
+    { "DebugGameNext", GTK_STOCK_GO_FORWARD, NULL, NULL, NULL,
+      G_CALLBACK (debug_game_next) },
+    { "DebugGamePrev", GTK_STOCK_GO_BACK, NULL, NULL, NULL,
+      G_CALLBACK (debug_game_prev) },
 #endif /* !HAVE_HILDON */
 #endif /* ENABLE_DEBUG_UI */
 
@@ -2137,7 +2260,13 @@ aisleriot_window_init (AisleriotWindow *window)
           "<menuitem action='DebugMoveNextScreen'/>"
           "<menuitem action='DebugDelayedMoveNextScreen'/>"
           "<menuitem action='DebugException'/>"
+          "<separator/>"
           "<menuitem action='DebugCycle'/>"
+          "<separator/>"
+          "<menuitem action='DebugGameFirst'/>"
+          "<menuitem action='DebugGamePrev'/>"
+          "<menuitem action='DebugGameNext'/>"
+          "<menuitem action='DebugGameLast'/>"
         "</menu>"
 #endif /* ENABLE_DEBUG_UI */
       "</menubar>"
@@ -2154,6 +2283,12 @@ aisleriot_window_init (AisleriotWindow *window)
 #ifndef HAVE_MAEMO
         "<toolitem action='LeaveFullscreen'/>"
 #endif
+#ifdef ENABLE_DEBUG_UI
+        "<toolitem action='DebugGameFirst'/>"
+        "<toolitem action='DebugGamePrev'/>"
+        "<toolitem action='DebugGameNext'/>"
+        "<toolitem action='DebugGameLast'/>"
+#endif
 #ifdef HAVE_MAEMO
         "<separator/>"
 #endif
@@ -2183,13 +2318,18 @@ aisleriot_window_init (AisleriotWindow *window)
 
   priv->game = aisleriot_game_new ();
 
-#ifdef HAVE_HILDON 
+  /* FIXMEchpe: case HAVE_HILDON && HAVE_RSVG ! */
+#ifdef HAVE_HILDON
   priv->scalable_cards = FALSE;
   priv->use_pixbuf_drawing = FALSE;
 #else
+#ifdef HAVE_RSVG
   /* Default to scalable */
   env = g_getenv ("AISLERIOT_CARDS_SCALABLE");
   priv->scalable_cards = env == NULL || g_ascii_strtoull (env, NULL, 0) != 0;
+#else
+  priv->scalable_cards = FALSE;
+#endif /* HAVE_RSVG */
 
   /* Default to pixbuf drawing */
   env = g_getenv ("AISLERIOT_PIXBUF_DRAWING");
@@ -2205,7 +2345,8 @@ aisleriot_window_init (AisleriotWindow *window)
     g_print ("Using pixbuf drawing method\n");
   else
     g_print ("Using pixmap drawing method\n");
-#endif
+#endif /* GNOME_ENABLE_DEBUG */
+
 #endif /* HAVE_MAEMO */
 
   priv->board = AISLERIOT_BOARD (aisleriot_board_new (priv->game, priv->scalable_cards));
@@ -2285,6 +2426,9 @@ aisleriot_window_init (AisleriotWindow *window)
   gtk_box_pack_start (GTK_BOX (priv->score_box), priv->score_label, FALSE, FALSE, 0);
   gtk_box_pack_end (GTK_BOX (statusbar_hbox), priv->score_box, FALSE, FALSE, 0);
 
+  aisleriot_util_add_atk_relation (label, priv->score_label, ATK_RELATION_LABEL_FOR);
+  aisleriot_util_add_atk_relation (priv->score_label, label, ATK_RELATION_LABELLED_BY);
+
   time_box = gtk_hbox_new (12, FALSE);
   label = gtk_label_new (_("Time:"));
   gtk_box_pack_start (GTK_BOX (time_box), label, FALSE, FALSE, 0);
@@ -2292,6 +2436,9 @@ aisleriot_window_init (AisleriotWindow *window)
   gtk_box_pack_start (GTK_BOX (time_box), priv->clock, FALSE, FALSE, 0);
   gtk_box_pack_end (GTK_BOX (statusbar_hbox), time_box, FALSE, FALSE, 0);
   gtk_widget_show_all (time_box);
+
+  aisleriot_util_add_atk_relation (label, priv->clock, ATK_RELATION_LABEL_FOR);
+  aisleriot_util_add_atk_relation (priv->clock, label, ATK_RELATION_LABELLED_BY);
 #endif /* !HAVE_HILDON */
 
   /* Load the UI after we've connected the statusbar,
