@@ -1,11 +1,21 @@
 # -*- coding: utf-8 -*-
+#
+# defaults.py.in sets many important default global variables
+# used throughout the game. Note that this file is processed by
+# automake to set prefix paths etc. Please keep defaults.py.in
+# in sync between glchess and gnome-sudoku. 
 
 import sys, os, os.path
 import errno
 import gettext 
 
-if sys.modules["glchess"].installed_mode:
-    APP_DATA_DIR     = os.path.join('/usr/local', 'share') 
+if (sys.modules.has_key('glchess') and
+    hasattr(sys.modules["glchess"],'installed_mode') and
+    sys.modules["glchess"].installed_mode):
+    # If the installed_mode attribute is not set, then we are
+    # importing from something other than the glchess script; we
+    # assume anyone importing in this way is doing testing etc.
+    APP_DATA_DIR     = os.path.join('/home/thomas/svn/test5', 'share') 
     ICON_DIR         = os.path.join(APP_DATA_DIR, 'pixmaps')
     TEXTURE_DIR      = os.path.join(ICON_DIR, 'glchess')
     GLADE_DIR        = os.path.join(APP_DATA_DIR, 'glchess')
@@ -18,11 +28,11 @@ if sys.modules["glchess"].installed_mode:
     UNFINISHED_FILE  = os.path.join(HISTORY_DIR, 'unfinished')
     LOCAL_AI_CONFIG  = os.path.join(DATA_DIR, 'ai.xml')
 else:
-    APP_DATA_DIR     = os.path.join('/home/jclinton/development/svn/GNOME_GAMES_2_21', 'glchess', 'data')
-    ICON_DIR         = os.path.join('/home/jclinton/development/svn/GNOME_GAMES_2_21', 'glchess', 'textures')
-    TEXTURE_DIR      = os.path.join('/home/jclinton/development/svn/GNOME_GAMES_2_21', 'glchess', 'textures')
-    GLADE_DIR        = os.path.join('/home/jclinton/development/svn/GNOME_GAMES_2_21', 'glchess', 'glade')
-    BASE_DIR         = os.path.join('/home/jclinton/development/svn/GNOME_GAMES_2_21', 'glchess', 'data')
+    APP_DATA_DIR     = os.path.join('/home/thomas/svn/gnome-games', 'glchess', 'data')
+    ICON_DIR         = os.path.join('/home/thomas/svn/gnome-games', 'glchess', 'textures')
+    TEXTURE_DIR      = os.path.join('/home/thomas/svn/gnome-games', 'glchess', 'textures')
+    GLADE_DIR        = os.path.join('/home/thomas/svn/gnome-games', 'glchess', 'glade')
+    BASE_DIR         = os.path.join('/home/thomas/svn/gnome-games', 'glchess', 'data')
     LOCALEDIR        = os.path.join(APP_DATA_DIR, 'locale')
     DATA_DIR         = os.path.expanduser('~/.gnome2/glchess/')
     LOG_DIR          = os.path.join(DATA_DIR, 'logs')
@@ -40,11 +50,11 @@ import gtk.glade
 gtk.glade.bindtextdomain(DOMAIN, LOCALEDIR)
 gtk.glade.textdomain(DOMAIN)
 
-VERSION   = "2.21.90"
+VERSION   = "2.21.91"
 APPNAME   = _("glChess")
 ICON_NAME = 'gnome-glchess'
 
-COPYRIGHT     = _('Copyright 2005-2007 Robert Ancell (and contributors)')
+COPYRIGHT     = _('Copyright 2005-2008 Robert Ancell (and contributors)')
 DESCRIPTION   = _('The 2D/3D chess game for GNOME. \n\nglChess is a part of GNOME Games.')
 WEBSITE       = 'http://www.gnome.org/projects/gnome-games/'
 WEBSITE_LABEL = _('GNOME Games web site')
