@@ -23,6 +23,7 @@ RULE_THREE_FOLD_REPETITION = 'THREE_FOLD_REPETITION'
 RULE_INSUFFICIENT_MATERIAL = 'INSUFFICIENT_MATERIAL'
 RULE_RESIGN                = 'RESIGN'
 RULE_DEATH                 = 'DEATH'
+RULE_AGREEMENT             = 'AGREEMENT'
 
 class ChessMove:
     """
@@ -100,6 +101,9 @@ class ChessPlayer:
         'move' is the record for this move (ChessMove).
         """
         pass
+    
+    def onPlayerStartTurn(self, player):
+        pass
 
     def onGameEnded(self, game):
         """Called when a chess game has ended.
@@ -171,6 +175,10 @@ class ChessPlayer:
         """Resign from the game"""
         self.__game.resign(self)
         
+    def claimDraw(self):
+        """Claim a draw"""
+        self.__game.claimDraw()
+
     def outOfTime(self):
         """Report this players timer has expired"""
         self.__game.outOfTime(self)
@@ -230,36 +238,36 @@ class ChessGameSANConverter(chess.san.SANConverter):
     for (a, b) in __typeToSAN.iteritems():
         __sanToType[b] = a
         
-    __board = None
-        
-    def __init__(self, board):
-        self.__board = board
+    def __init__(self, board, moveNumber):
+        self.board = board
+        self.moveNumber = moveNumber
         chess.san.SANConverter.__init__(self)
     
     def decodeSAN(self, colour, move):
         (start, end, result, promotionType) = chess.san.SANConverter.decodeSAN(self, self.__colourToSAN[colour], move)
         return (start, end, self.__sanToType[promotionType])
     
-    def encodeSAN(self, start, end, promotionType):
+    def encodeSAN(self, start, end, isTake, promotionType):
         if promotionType is None:
             promotion = self.QUEEN
         else:
             promotion = self.__typeToSAN[promotionType]
-        return chess.san.SANConverter.encode(self, start, end, promotion)
+        return self.encode(start, end, isTake, promotion)
 
     def getPiece(self, location):
         """Called by chess.san.SANConverter"""
-        piece = self.__board.getPiece(location)
+        piece = self.board.getPiece(location, self.moveNumber)
         if piece is None:
             return None
         return (self.__colourToSAN[piece.getColour()], self.__typeToSAN[piece.getType()])
     
     def testMove(self, colour, start, end, promotionType, allowSuicide = False):
         """Called by chess.san.SANConverter"""
-        move = self.__board.testMove(self.__sanToColour[colour], start, end, self.__sanToType[promotionType], allowSuicide)
+        move = self.board.testMove(self.__sanToColour[colour], start, end,
+                                     self.__sanToType[promotionType], allowSuicide, self.moveNumber)
         if move is None:
             return False
-        
+
         if move.opponentInCheck:
             if not move.opponentCanMove:
                 return chess.san.SANConverter.CHECKMATE
@@ -276,10 +284,7 @@ class ChessGame:
     __spectators = None
     
     # The board to move on
-    __board = None
-    
-    # SAN en/decoders
-    __sanConverter = None
+    board = None
     
     # The game state (started and player to move)
     __started = False
@@ -300,8 +305,7 @@ class ChessGame:
         """Game constructor"""
         self.__players = []
         self.__spectators = []
-        self.__board = ChessGameBoard(self)
-        self.__sanConverter = ChessGameSANConverter(self.__board)
+        self.board = ChessGameBoard(self)
         self.__moves = []
         self.__queuedCalls = []
         
@@ -313,7 +317,7 @@ class ChessGame:
         Returns a dictionary of the alive pieces (board.ChessPiece) keyed by location.
         Raises an IndexError exception if moveNumber is invalid.
         """
-        return self.__board.getAlivePieces(moveNumber)
+        return self.board.getAlivePieces(moveNumber)
     
     def getDeadPieces(self, moveNumber = -1):
         """Get the dead pieces from the game.
@@ -323,7 +327,7 @@ class ChessGame:
         Returns a list of the pieces (board.ChessPiece) in the order they were killed.
         Raises an IndexError exception if moveNumber is invalid.
         """
-        return self.__board.getDeadPieces(moveNumber)
+        return self.board.getDeadPieces(moveNumber)
     
     def setTimers(self, whiteTimer, blackTimer):
         """
@@ -409,14 +413,15 @@ class ChessGame:
         self.startLock()
         
         # Get the next player to move
-        self.__currentPlayer._setReadyToMove(True)
+        if self.result == RESULT_IN_PROGRESS:
+            self.__currentPlayer._setReadyToMove(True)
 
         self.endLock()
         
     def getSquareOwner(self, coord):
         """TODO
         """
-        piece = self.__board.getPiece(coord)
+        piece = self.board.getPiece(coord)
         if piece is None:
             return None
         
@@ -448,7 +453,7 @@ class ChessGame:
         else:
             assert(False)
 
-        move = self.__board.testMove(colour, start, end, promotionType = promotionType)
+        move = self.board.testMove(colour, start, end, promotionType = promotionType)
 
         return move is not None
     
@@ -485,6 +490,10 @@ class ChessGame:
     def _move(self, player, move):
         """
         """
+        if self.result != RESULT_IN_PROGRESS:
+            print 'Game completed'
+            return
+        
         if self.__currentPlayer is self.__whitePlayer:
             colour = chess.board.WHITE
         else:
@@ -494,15 +503,15 @@ class ChessGame:
         try:
             (start, end, _, _, promotionType, _) = chess.lan.decode(colour, move)
         except chess.lan.DecodeError, e:
+            converter = ChessGameSANConverter(self.board, len(self.__moves))
             try:
-                (start, end, promotionType) = self.__sanConverter.decodeSAN(colour, move)
+                (start, end, promotionType) = converter.decodeSAN(colour, move)
             except chess.san.Error, e:
                 print 'Invalid move: ' + move
                 return
 
         # Only use promotion type if a pawn move to far file
-        victim = self.__board.getPiece(end)
-        piece = self.__board.getPiece(start)
+        piece = self.board.getPiece(start)
         promotion = None
         if piece is not None and piece.getType() is chess.board.PAWN:
             if colour is chess.board.WHITE:
@@ -512,14 +521,19 @@ class ChessGame:
                 if end[1] == '1':
                     promotion = promotionType
 
-        # Re-encode for storing and reporting
-        sanMove = self.__sanConverter.encodeSAN(start, end, promotionType)
-        canMove = chess.lan.encode(colour, start, end, promotionType = promotion)
-        moveResult = self.__board.movePiece(colour, start, end, promotionType)
-
+        moveResult = self.board.movePiece(colour, start, end, promotionType)
         if moveResult is None:
             print 'Illegal move: ' + str(move)
             return
+        
+        # Re-encode for storing and reporting
+        canMove = chess.lan.encode(colour, start, end, promotionType = promotion)
+        converter = ChessGameSANConverter(self.board, len(self.__moves))
+        try:
+            sanMove = converter.encodeSAN(start, end, moveResult.victim != None, promotionType)
+        except chess.san.Error:
+            # If for some reason we couldn't make the SAN move the use the CAN move instead
+            sanMove = canMove
 
         m = ChessMove()
         if len(self.__moves) == 0:
@@ -528,7 +542,7 @@ class ChessGame:
             m.number = self.__moves[-1].number + 1
         m.player              = self.__currentPlayer
         m.piece               = piece
-        m.victim              = victim
+        m.victim              = moveResult.victim
         m.start               = start
         m.end                 = end
         m.canMove             = canMove
@@ -585,9 +599,13 @@ class ChessGame:
             self.__currentPlayer = self.__whitePlayer
         
         self.startLock()
+        
+        # Inform other players of the result
+        for player in self.__players:
+            player.onPlayerStartTurn(self.__currentPlayer)
 
         # Notify the next player they can move
-        if self.__started is True:
+        if self.__started is True and self.result == RESULT_IN_PROGRESS:
             self.__currentPlayer._setReadyToMove(True)
 
         self.endLock()
@@ -597,24 +615,31 @@ class ChessGame:
         
         'player' is the player resigning.
         """
-        if player is not self.__currentPlayer:
-            print 'Player attempted to resign out of turn'
-            return
-
-        # See if the resignation can be done without requesting
         rule = RULE_RESIGN
+        if player is self.__whitePlayer:
+            self.endGame(RESULT_BLACK_WINS, rule)
+        else:
+            self.endGame(RESULT_WHITE_WINS, rule)
+            
+    def claimDraw(self):
+        """
+        """
+        # TODO: Penalise if make an incorrect attempt
         try:
             move = self.__moves[-1]
         except IndexError:
-            pass
+            return False
         else:
             if move.fiftyMoveRule:
                 rule = RULE_FIFTY_MOVES
             elif move.threeFoldRepetition:
                 rule = RULE_THREE_FOLD_REPETITION
+            else:
+                return False
 
         self.endGame(RESULT_DRAW, rule)
-        
+        return True
+
     def killPlayer(self, player):
         """Report a player has died
         
@@ -639,6 +664,7 @@ class ChessGame:
     def endGame(self, result, rule):
         self.result = result
         self.rule = rule
+        self.__currentPlayer._setReadyToMove(False)
         for player in self.__players:
             player.onGameEnded(self)
 
@@ -670,7 +696,7 @@ class ChessGame:
         for file in '12345678':
             for rank in 'abcdefgh':
                 coord = rank + file
-                piece = self.__board.getPiece(coord)
+                piece = self.board.getPiece(coord)
                 if piece is None:
                     continue
 
