@@ -12,7 +12,7 @@ import gtk.gdk
 
 import gtkui
 import glchess.ui
-        
+
 class GtkServerList:
     __gui = None
 
@@ -75,13 +75,6 @@ class GtkServerList:
 class GtkNewGameDialog:
     """
     """    
-    # The main UI and the ???
-    __mainUI = None
-    __gui = None
-
-    __customName = False
-    __checking = False
-    
     def __init__(self, mainUI, aiModel, game = None):
         """Constructor for a new game dialog.
         
@@ -93,13 +86,17 @@ class GtkNewGameDialog:
         self.game = game
         
         self.__checking = True
-        
+        self.__customName = False
+
         # Load the UI
         self.__gui = gtkui.loadGladeFile('new_game.glade', 'new_game_dialog')
         self.__gui.signal_autoconnect(self)
-        
+
+        self.window = self.__gui.get_widget('new_game_dialog')
+        self.window.set_transient_for(mainUI.mainWindow)
+
         # Set style of error panel
-        self.__gui.get_widget('info_box').set_style(mainUI.tooltipStyle)
+        mainUI.setTooltipStyle(self.__gui.get_widget('info_box'))
         
         # Make all the labels the same width
         group = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
@@ -166,19 +163,15 @@ class GtkNewGameDialog:
         widget.set_active(0)
 
         # Create the model for difficulty options
-        levelModel = gtk.ListStore(str, gtk.gdk.Pixbuf, str)
+        levelModel = gtk.ListStore(str, str, str)
         levels = [('easy',   _('Easy'),   'weather-few-clouds'),
                   ('normal', _('Normal'), 'weather-overcast'),
                   ('hard',   _('Hard'),   'weather-storm')]
         iconTheme = gtk.icon_theme_get_default()
-        for (key, label, iconName) in levels:
-            try:
-                icon = iconTheme.load_icon(iconName, 24, gtk.ICON_LOOKUP_USE_BUILTIN)
-            except gobject.GError:
-                icon = None
+        for (key, label, icon) in levels:
             iter = levelModel.append()
             levelModel.set(iter, 0, key, 1, icon, 2, label)
-        
+
         # Set the difficulty settings
         for name in ['black_difficulty_combo', 'white_difficulty_combo']:
             widget = self.__gui.get_widget(name)
@@ -189,7 +182,7 @@ class GtkNewGameDialog:
             
             cell = gtk.CellRendererPixbuf()
             widget.pack_start(cell, False)
-            widget.add_attribute(cell, 'pixbuf', 1)
+            widget.add_attribute(cell, 'icon-name', 1)
             
             cell = gtk.CellRendererText()
             widget.pack_start(cell, False)
@@ -208,7 +201,7 @@ class GtkNewGameDialog:
             
             cell = gtk.CellRendererPixbuf()
             widget.pack_start(cell, False)
-            widget.add_attribute(cell, 'pixbuf', 1)
+            widget.add_attribute(cell, 'icon-name', 1)
             
             cell = gtk.CellRendererText()
             widget.pack_start(cell, False)
@@ -245,8 +238,8 @@ class GtkNewGameDialog:
             # TODO: Others
             
             # Change title for loaded games
-            if len(g.path) > 0:
-                self.__gui.get_widget('new_game_dialog').set_title(_('Configure loaded game (%i moves)') % len(g.moves))
+            if g.path is not None:
+                self.window.set_title(_('Configure loaded game (%i moves)') % len(g.moves))
 
         # Display warning if missing the AIs
         if len(errorText) > 0:
@@ -255,7 +248,7 @@ class GtkNewGameDialog:
             self.__gui.get_widget('info_box').show()
 
         # Show the dialog
-        self.__gui.get_widget('new_game_dialog').show()
+        self.window.present()
         self.__checking = False
         self.__testReady()
         
@@ -304,6 +297,8 @@ class GtkNewGameDialog:
         if self.__customName:
             name = self.__gui.get_widget('game_name_entry').get_text()
             if len(name) == 0:
+                # Next time the something changes generate a name
+                self.__customName = False
                 ready = False
 
         # Name the game based on the players
@@ -312,7 +307,7 @@ class GtkNewGameDialog:
             blackName = self.__getComboData(self.__gui.get_widget('black_type_combo'), 2)
             format = _('%(white)s versus %(black)s')
             self.__gui.get_widget('game_name_entry').set_text(format % {'white': whiteName, 'black': blackName})
-            
+
         # Disable difficulty for human players
         whiteType = self.__getComboData(self.__gui.get_widget('white_type_combo'), 0)
         blackType = self.__getComboData(self.__gui.get_widget('black_type_combo'), 0)
@@ -365,22 +360,12 @@ class GtkNewGameDialog:
         
     # Gtk+ signal handlers
     
-    def _on_info_box_expose_event(self, widget, event):
-        allocation = widget.allocation
-        widget.style.paint_flat_box(widget.window, gtk.STATE_NORMAL, gtk.SHADOW_OUT, None, widget, "tooltip",
-                                    allocation.x, allocation.y, allocation.width, allocation.height)
-
-        # The first draw is corrupt for me so draw it twice.
-        # Bonus points to anyone who tracks down the problem and fixes it
-        #if self.firstExpose:
-        #    self.firstExpose = False
-        #    widget.queue_draw()
-    
     def _on_game_name_edited(self, widget):
         """Gtk+ callback"""
         if self.__checking:
             return
-        self.__customName = len(widget.get_text()) != 0
+        self.__customName = True
+        self.__testReady()
 
     def _on_time_changed(self, widget):
         """Gtk+ callback"""
@@ -395,276 +380,15 @@ class GtkNewGameDialog:
         """Gtk+ callback"""
         self.__testReady()
 
-    def _on_response(self, dialog, response_id):
+    def _on_response(self, dialog, responseId):
         """Gtk+ callback"""
-        if response_id == gtk.RESPONSE_OK:
+        if responseId == gtk.RESPONSE_OK:
             self.__startGame()
         dialog.destroy()
 
-class GtkNetworkGameDialog(glchess.ui.NetworkController):
-    """
-    """
-    # The main UI and the ???
-    __mainUI = None
-    __gui = None
-
-    __customName = False
-    __checking = False
-    
-    def __init__(self, mainUI, feedback, aiModel):
-        """Constructor for a new game dialog.
-        
-        'mainUI' is the main UI.
-        'feedback' is the object to feedback events with.
-        'aiModel' is the AI models to use.
-        """
-        self.__mainUI = mainUI
-        self.feedback = feedback
-        
-        # Load the UI
-        self.__gui = gtkui.loadGladeFile('network_game.glade', 'network_game_dialog')
-        self.__gui.signal_autoconnect(self)
-        
-        # Make all the entries the same width
-        group = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
-        for name in ['label1', 'label2', 'label3']:
-            widget = self.__gui.get_widget(name)
-            group.add_widget(widget)
-
-        # Make all the images the same width
-        group = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
-        i = 1
-        while True:
-            widget = self.__gui.get_widget('image%i' % i)
-            if widget is None:
-                break
-            group.add_widget(widget)
-            i += 1
-
-        # Create the model for difficulty options
-        levelModel = gtk.ListStore(str, gtk.gdk.Pixbuf, str)
-        levels = [('easy',   _('Easy'),   'weather-few-clouds'),
-                  ('normal', _('Normal'), 'weather-overcast'),
-                  ('hard',   _('Hard'),   'weather-storm')]
-        iconTheme = gtk.icon_theme_get_default()
-        for (key, label, iconName) in levels:
-            try:
-                icon = iconTheme.load_icon(iconName, 24, gtk.ICON_LOOKUP_USE_BUILTIN)
-            except gobject.GError:
-                icon = None
-            iter = levelModel.append()
-            levelModel.set(iter, 0, key, 1, icon, 2, label)
-        
-        # Set the difficulty settings
-        widget = self.__gui.get_widget('difficulty_combo')
-        widget.set_model(levelModel)
-            
-        cell = gtk.CellRendererPixbuf()
-        widget.pack_start(cell, False)
-        widget.add_attribute(cell, 'pixbuf', 1)
-            
-        cell = gtk.CellRendererText()
-        widget.pack_start(cell, False)
-        widget.add_attribute(cell, 'text', 2)
-            
-        widget.set_active(1)
-        
-        # Make all the AI combo boxes use one list of AI types
-        widget = self.__gui.get_widget('player_type_combo')
-        widget.set_model(aiModel)
-            
-        cell = gtk.CellRendererPixbuf()
-        widget.pack_start(cell, False)
-        widget.add_attribute(cell, 'pixbuf', 1)
-            
-        cell = gtk.CellRendererText()
-        widget.pack_start(cell, False)
-        widget.add_attribute(cell, 'text', 2)
-
-        widget.set_active(0)
-
-        # Configure player
-        #try:
-        #    whiteType = glchess.config.get('new_game_dialog/white/type')
-        #    whiteLevel = glchess.config.get('new_game_dialog/white/difficulty')
-        #except glchess.config.Error:
-        #    pass
-        #else:
-        #    self.__setCombo('white_type_combo', whiteType)
-        #    self.__setCombo('white_difficulty_combo', whiteLevel)
-        #    self.__setCombo('black_type_combo', blackType)
-        #    self.__setCombo('black_difficulty_combo', blackLevel)
-        
-        # Create the model for the advertised games
-        self.gameModel = gtk.ListStore(gobject.TYPE_PYOBJECT, str, str, str)
-        view = self.__gui.get_widget('game_treeview')
-        view.set_model(self.gameModel)
-
-        cell = gtk.CellRendererText()
-        column = gtk.TreeViewColumn('Player', cell)
-        column.add_attribute(cell, 'text', 1)
-        view.append_column(column)
-
-        cell = gtk.CellRendererText()
-        column = gtk.TreeViewColumn('Rating', cell)
-        column.add_attribute(cell, 'text', 2)
-        view.append_column(column)
-
-        cell = gtk.CellRendererText()
-        column = gtk.TreeViewColumn('Variant', cell)
-        column.add_attribute(cell, 'text', 3)
-        view.append_column(column)
-
-        # Create styles for the buffer
-        buffer = self.__gui.get_widget('chat_textview').get_buffer()
-        buffer.create_tag('chat', family='Monospace')
-        #buffer.create_tag('output', family='Monospace', weight = pango.WEIGHT_BOLD)
-        #buffer.create_tag('move', family='Monospace', foreground = 'blue')
-        buffer.create_tag('info', family='Monospace', foreground = 'gray')
-        #buffer.create_tag('error', family='Monospace', foreground = 'red')
-
-        # Show the dialog
-        self.__testReady()
-        self.__gui.get_widget('network_game_dialog').show()
-
-    # Extended methods
-    
-    def addText(self, text, style):
-        """Called by glchess.ui.UIController"""
-        buffer = self.__gui.get_widget('chat_textview').get_buffer()
-        buffer.insert_with_tags_by_name(buffer.get_end_iter(), text, style)
-
-    def addAdvert(self, title, rating, variant, advert):
-        """Called by glchess.ui.UIController"""
-        iter = self.gameModel.append()
-        self.gameModel.set(iter, 0, advert, 1, title, 2, rating, 3, variant)
-        
-    def removeAdvert(self, advert):
-        """Called by glchess.ui.UIController"""
-        iter = self.gameModel.get_iter_first()
-        while iter:
-            if self.gameModel.get_value(iter, 0) is advert:
-                self.gameModel.remove(iter)
-                return
-            iter = self.gameModel.iter_next(iter)
-        print 'WARNING: Cannot find advert to remove'
-
-    def requestGame(self, gameName):
-        """Called by glchess.ui.UIController"""
-        # TEMP
-        self.feedback.acceptGame()
-
-    # Private methods
-    
-    def __setCombo(self, comboName, key):
-        widget = self.__gui.get_widget(comboName)
-        iter = self.__getIter(widget.get_model(), key)
-        if iter is not None:
-            widget.set_active_iter(iter)
-    
-    def __getIter(self, model, key, default = None):
-        """
-        """
-        iter = model.get_iter_first()
-        while iter:
-            if model.get_value(iter, 0) == key:
-                return iter
-
-            iter = model.iter_next(iter)
-        return default
-                
-    def __getComboData(self, comboBox, index):
-        """
-        """
-        model = comboBox.get_model()
-        iter = comboBox.get_active_iter()
-        if iter is None:
-            return None
-        
-        data = model.get(iter, index)
-        return data[0]
-
-    def __testReady(self):
-        if self.__checking:
-            return
-        self.__checking = True
-        ready = True
-           
-        # Disable difficulty for human players
-        playerType = self.__getComboData(self.__gui.get_widget('player_type_combo'), 0)
-        self.__gui.get_widget('difficulty_combo').set_sensitive(playerType != None)
-
-        # Get server/game
-        # FIXME
-        ready = False
-
-        # Can only click Connect if have enough information
-        self.__gui.get_widget('connect_button').set_sensitive(ready)
-        self.__checking = False
-        
-    def __startGame(self):
-        game = glchess.ui.Game()
-        game.name = self.__gui.get_widget('game_name_entry').get_text()
-        game.allowSpectators = True
-        
-        # Get the players
-        game.white.type  = self.__getComboData(self.__gui.get_widget('white_type_combo'), 0)
-        if game.white.type is '':
-            game.white.name = _('White')
-        else:
-            game.white.name = self.__getComboData(self.__gui.get_widget('white_type_combo'), 2)
-        game.white.level = self.__getComboData(self.__gui.get_widget('white_difficulty_combo'), 0)
-        game.black.type  = self.__getComboData(self.__gui.get_widget('black_type_combo'), 0)
-        if game.black.type == '':
-            game.black.name = _('Black')
-        else:
-            game.black.name = self.__getComboData(self.__gui.get_widget('black_type_combo'), 2)
-        game.black.level = self.__getComboData(self.__gui.get_widget('black_difficulty_combo'), 0)
-
-        game.duration = self.__getComboData(self.__gui.get_widget('time_combo'), 1)
-        if game.duration < 0:
-            multplier = self.__getComboData(self.__gui.get_widget('custom_time_units_combo'), 1)
-            game.duration = self.__getComboData(self.__gui.get_widget('custom_time_spin'), 1) * multiplier
-            
-        # Save properties
-        glchess.config.set('new_game_dialog/white/type', game.white.type)
-        glchess.config.set('new_game_dialog/white/difficulty', game.white.level)
-        glchess.config.set('new_game_dialog/black/type', game.black.type)
-        glchess.config.set('new_game_dialog/black/difficulty', game.black.level)
-
-        # Inform the child class
-        self.__mainUI.feedback.onGameStart(game)
-        
-    # Gtk+ signal handlers
-    
-    def _on_game_name_edited(self, widget):
-        """Gtk+ callback"""
-        if self.__checking:
-            return
-        self.__customName = len(widget.get_text()) != 0
-
-    def _on_properties_changed(self, widget, *data):
-        """Gtk+ callback"""
-        self.__testReady()
-        
-    def _on_chat_entry_activate(self, widget):
-        """Gtk+ callback"""
-        text = widget.get_text()
-        widget.set_text('')
-        self.feedback.sendCommand(text)
-
-    def _on_response(self, widget, response_id):
-        """Gtk+ callback"""
-        if response_id == gtk.RESPONSE_OK:
-            self.__startGame()
-        self.__gui.get_widget('new_game_dialog').destroy()
-
 class GtkLoadGameDialog:
     """
     """    
-    __mainUI = None
-    __gui = None
-    
     def __init__(self, mainUI):
         """
         """
@@ -674,8 +398,11 @@ class GtkLoadGameDialog:
         self.__gui = gtkui.loadGladeFile('load_game.glade')
         self.__gui.signal_autoconnect(self)
         
+        self.window = self.__gui.get_widget('game_load_dialog')
+        self.window.set_transient_for(mainUI.mainWindow)
+        
         # Set style of error panel
-        self.__gui.get_widget('error_box').set_style(mainUI.tooltipStyle)
+        mainUI.setTooltipStyle(self.__gui.get_widget('error_box'))
         
         fileChooser = self.__gui.get_widget('filechooserwidget')
         
@@ -697,8 +424,7 @@ class GtkLoadGameDialog:
         allFilter.add_pattern('*')
         fileChooser.add_filter(allFilter)
         
-        dialog = self.__gui.get_widget('game_load_dialog')
-        dialog.show()
+        self.window.present()
         
     def __getFilename(self):
         """Get the currently selected filename.
@@ -719,14 +445,16 @@ class GtkLoadGameDialog:
         
     def _on_file_activated(self, widget):
         """Gtk+ callback"""
-        self._on_response(self.__gui.get_widget('game_load_dialog'), gtk.RESPONSE_OK)
+        self._on_response(self.window, gtk.RESPONSE_OK)
 
     def _on_response(self, dialog, responseId):
         """Gtk+ callback"""
         chooser= self.__gui.get_widget('filechooserwidget')
 
         if responseId == gtk.RESPONSE_OK or responseId == gtk.RESPONSE_YES:
-            glchess.config.set('load_directory', chooser.get_current_folder())
+            folder = chooser.get_current_folder()
+            if folder is not None:
+                glchess.config.set('load_directory', folder)
             error = self.__mainUI.feedback.loadGame(self.__getFilename(), responseId == gtk.RESPONSE_YES)
             if error is not None:
                 self.firstExpose = True
@@ -736,79 +464,10 @@ class GtkLoadGameDialog:
                 return
 
         dialog.destroy()
-
-    def _on_error_box_expose_event(self, widget, event):
-        """Gtk+ callback"""
-        allocation = widget.allocation
-        widget.style.paint_flat_box(widget.window, gtk.STATE_NORMAL, gtk.SHADOW_OUT, None, widget, "tooltip",
-                                    allocation.x, allocation.y, allocation.width, allocation.height)
-
-        # The first draw is corrupt for me so draw it twice.
-        # Bonus points to anyone who tracks down the problem and fixes it
-        if self.firstExpose:
-            self.firstExpose = False
-            widget.queue_draw()
-
-class SaveDialog:
-    """
-    """
-
-    def __init__(self, ui):
-        """Constructor"""
-        self.ui = ui
-        self.model = gtk.ListStore(gobject.TYPE_PYOBJECT, gobject.TYPE_BOOLEAN, str)
-        
-        self.view = ui._gui.get_widget('save_games_treeview')
-        self.view.set_model(self.model)
-        
-        selection = self.view.get_selection()
-        selection.set_mode(gtk.SELECTION_NONE)
-
-        cell = gtk.CellRendererToggle()
-        cell.set_property('activatable', True)
-        cell.connect('toggled', self._toggle_cb)
-        column = gtk.TreeViewColumn('Save', cell)
-        column.add_attribute(cell, 'active', 1)
-        self.view.append_column(column)
-        
-        cell = gtk.CellRendererText()
-        column = gtk.TreeViewColumn('Game', cell)
-        column.add_attribute(cell, 'text', 2)
-        self.view.append_column(column)
-        
-    def setViews(self, views):
-        """
-        """
-        self.model.clear()
-        for view in views:
-            iter = self.model.append()
-            self.model.set(iter, 0, view, 1, True, 2, view.title)
-
-    def setVisible(self, isVisible):
-        """
-        """
-        widget = self.ui._gui.get_widget('save_dialog')
-        if isVisible:
-            widget.show_all()
-        else:
-            widget.hide()
-            
-    def _toggle_cb(self, widget, path):
-        """Gtk+ callback"""
-        self.model[path][1] = not self.model[path][1]
         
 class GtkSaveGameDialog:
     """
-    """    
-    # The main UI
-    __mainUI = None
-
-    # The view that is being saved
-    __view = None
-    
-    # The GUI
-    __gui = None
-    
+    """
     def __init__(self, mainUI, view, path = None):
         """
         """
@@ -820,9 +479,10 @@ class GtkSaveGameDialog:
         self.__gui.signal_autoconnect(self)
         
         # Set style of error panel
-        self.__gui.get_widget('error_box').set_style(mainUI.tooltipStyle)
+        mainUI.setTooltipStyle(self.__gui.get_widget('error_box'))
 
-        dialog = self.__gui.get_widget('save_dialog')
+        self.window = self.__gui.get_widget('save_dialog')
+        self.window.set_transient_for(mainUI.mainWindow)
         chooser = self.__gui.get_widget('filechooser')
         
         try:
@@ -845,22 +505,10 @@ class GtkSaveGameDialog:
         
         if path is not None:
             chooser.set_current_name(path)
-
-    def _on_error_box_expose_event(self, widget, event):
-        """Gtk+ callback"""
-        allocation = widget.allocation
-        widget.style.paint_flat_box(widget.window, gtk.STATE_NORMAL, gtk.SHADOW_OUT, None, widget, "tooltip",
-                                    allocation.x, allocation.y, allocation.width, allocation.height)
-
-        # The first draw is corrupt for me so draw it twice.
-        # Bonus points to anyone who tracks down the problem and fixes it
-        if self.firstExpose:
-            self.firstExpose = False
-            widget.queue_draw()
         
     def _on_file_activated(self, widget):
         """Gtk+ callback"""
-        self._on_response(self.__gui.get_widget('save_dialog'), gtk.RESPONSE_OK)
+        self._on_response(self.window, gtk.RESPONSE_OK)
 
     def __setError(self, title, content):
         """
@@ -884,8 +532,10 @@ class GtkSaveGameDialog:
                 fname += '.pgn'
 
             # Save the directory we used
-            glchess.config.set('save_directory', chooser.get_current_folder())
-        
+            folder = chooser.get_current_folder()
+            if folder is not None:
+                glchess.config.set('save_directory', folder)
+
             error = self.__mainUI._saveView(self.__view, fname)
             if error is not None:
                 self.__setError(_('Unabled to save game'), error)
@@ -895,3 +545,141 @@ class GtkSaveGameDialog:
 
         dialog.destroy()
 
+class GtkPreferencesDialog:
+    """
+    """
+    def __init__(self, mainUI):
+        """Constructor for the preferences dialog.
+        
+        'mainUI' is the main UI.
+        """
+        # Load the UI
+        self.__gui = gtkui.loadGladeFile('preferences.glade', 'preferences')
+        self.__gui.signal_autoconnect(self)
+        
+        self.__gui.get_widget('preferences').set_transient_for(mainUI.mainWindow)
+
+        # Load preferences for move_format
+        moveFormat = glchess.config.get('move_format')
+        moveModel = gtk.ListStore(str, str)
+        widget = self.__gui.get_widget('move_format_combo')
+        widget.set_model(moveModel)
+        move_formats = [('human', _('Human')),
+                        ('lan', _('Long Algebraic')),
+                        ('san', _('Standard Algebraic'))]
+        for (key, label) in move_formats:
+            iter = moveModel.append()
+            if key == moveFormat:
+                widget.set_active_iter(iter)
+            moveModel.set(iter, 0, label, 1, key)
+
+        # Load preferences for board orientation
+        boardView = glchess.config.get('board_view')
+        boardModel = gtk.ListStore(str, str)
+        widget = self.__gui.get_widget('board_combo')
+        widget.set_model(boardModel)
+        view_list = [('white', _('White Side')),
+                     ('black', _('Black Side')),
+                     ('human', _('Human Side')),
+                     ('current', _('Current Player'))]
+        for (key, label) in view_list:
+            iter = boardModel.append()
+            if key == boardView:
+                widget.set_active_iter(iter)
+            boardModel.set(iter, 0, label, 1, key)
+
+        # Load preferences for promotion type
+        promotionType = glchess.config.get('promotion_type')
+        promotionModel = gtk.ListStore(str, str)
+        widget = self.__gui.get_widget('promotion_type_combo')
+        widget.set_model(promotionModel)
+        promotion_list = [('queen', _('Queen')),
+                          ('knight', _('Knight')),
+                          ('rook', _('Rook')),
+                          ('bishop', _('Bishop'))]
+        for (key, label) in promotion_list:
+            iter = promotionModel.append()
+            if key == promotionType:
+                widget.set_active_iter(iter)
+            promotionModel.set(iter, 0, label, 1, key)
+
+        # Load preferences for View settings.
+        pref = glchess.config.get('show_3d')
+        widget = self.__gui.get_widget('show_3d')
+        widget.set_active(pref)
+
+        pref = glchess.config.get('show_toolbar')
+        widget = self.__gui.get_widget('show_toolbar')
+        widget.set_active(pref)
+
+        pref = glchess.config.get('show_history')
+        widget = self.__gui.get_widget('show_history')
+        widget.set_active(pref)
+
+        pref = glchess.config.get('show_move_hints')
+        widget = self.__gui.get_widget('show_move_hints')
+        widget.set_active(pref)
+
+        pref = glchess.config.get('show_numbering')
+        widget = self.__gui.get_widget('show_numbering')
+        widget.set_active(pref)
+
+    def setVisible(self, isVisible):
+        window = self.__gui.get_widget('preferences')
+        if isVisible:
+            window.present()
+        else:
+            window.hide()
+
+    def _on_response(self, dialog, responseId):
+        """Gtk+ callback"""
+        self.setVisible(False)
+        
+    def _on_delete(self, dialog, event):
+        """Gtk+ callback"""
+        self.setVisible(False)
+        return True
+
+    def _on_move_format_combo_changed(self, widget):
+        """Gtk+ callback"""
+        model = widget.get_model()
+        iter = widget.get_active_iter()
+        data = model.get(iter, 1)
+        if data[0] is not None:
+            glchess.config.set('move_format', data[0])
+
+    def _on_board_combo_changed(self, widget):
+        """Gtk+ callback"""
+        model = widget.get_model()
+        iter = widget.get_active_iter()
+        data = model.get(iter, 1)
+        if data[0] is not None:
+            glchess.config.set('board_view', data[0])
+
+    def _on_promotion_type_combo_changed(self, widget):
+        """Gtk+ callback"""
+        model = widget.get_model()
+        iter = widget.get_active_iter()
+        data = model.get(iter, 1)
+        if data[0] is not None:
+            glchess.config.set('promotion_type', data[0])
+
+    def _on_3d_view_activate(self, widget):
+        """Gtk+ callback"""
+        glchess.config.set('show_3d', widget.get_active())
+
+    def _on_show_toolbar_activate(self, widget):
+        """Gtk+ callback"""
+        glchess.config.set('show_toolbar', widget.get_active())
+
+    def _on_show_history_activate(self, widget):
+        """Gtk+ callback"""
+        glchess.config.set('show_history', widget.get_active())
+
+    def _on_move_hints_activate(self, widget):
+        """Gtk+ callback"""
+        glchess.config.set('show_move_hints', widget.get_active())
+
+    def _on_board_numbering_activate(self, widget):
+        """Gtk+ callback"""
+        glchess.config.set('show_numbering', widget.get_active())
