@@ -8,6 +8,8 @@ import ggz
 import ui
 import game
 
+_ = gettext.gettext
+
 class GGZServer:
     pass
 
@@ -57,19 +59,21 @@ class GGZChannel(ggz.Channel):
         self.ui = ui
         self.feedback = feedback
         self.socket = None
+        self.fd = -1
         
     def logXML(self, text):
         self.logWindow.addText(text, 'input')
-        
+
     def logBinary(self, data):
         self.logWindow.addBinary(data, 'input')
 
     def connect(self, host, port):
         self.close()
         self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        self.fd = self.socket.fileno()
         self.socket.setblocking(False)
-        self.ui.application.ioHandlers[self.socket.fileno()] = self
-        self.ui.controller.watchFileDescriptor(self.socket.fileno())
+        self.ui.application.ioHandlers[self.fd] = self
+        self.ui.controller.watchFileDescriptor(self.fd)
         
         try:
             self.socket.connect((host, port))
@@ -99,9 +103,10 @@ class GGZChannel(ggz.Channel):
     def close(self):
         if self.socket is None:
             return
-        # FIXME: Unregister fd events
+        self.ui.controller.unwatchFileDescriptor(self.fd)
         self.socket.close()
         self.socket = None
+        self.fd = -1
         
     def read(self):
         try:
@@ -162,10 +167,11 @@ class GGZConnection(ggz.ClientFeedback):
         self.dialog.controller.clearError()
 
     def onDisconnected(self):
-        self.dialog.controller.setError('Disconnected', 'You have been disconnected from the server')
+        self.dialog.controller.setError(_('Disconnected'), _('You have been disconnected from the server'))
 
     def openChannel(self, feedback):
         print 'Open Channel'
+        self.setBusy(True)
         socket = GGZChannel(self.dialog.ui, feedback)
         socket.connect(self.profile.host, self.profile.port)
         return socket
@@ -210,7 +216,7 @@ class GGZConnection(ggz.ClientFeedback):
             return
         description = table.description
         if len(description) == 0:
-            description = gettext.gettext('No description')
+            description = _('No description')
         nUsed = 0
         for seat in table.seats:
             if seat.type == 'bot' or seat.user != '':
@@ -230,7 +236,7 @@ class GGZConnection(ggz.ClientFeedback):
 
     def onJoin(self, table, isSpectator, channel):
         self.dialog.controller.joinTable(table)
-        g = GGZChess(self.dialog.ui, channel)
+        g = GGZChess(self.dialog.game, channel)
         return g
 
     def onLeave(self, reason):
@@ -250,8 +256,8 @@ class GGZChess:
     """
     """
     
-    def __init__(self, ui, channel):
-        self.ui = ui
+    def __init__(self, game, channel):
+        self.game = game
         self.channel = channel
         self.protocol = ggz.Chess(self)
 
@@ -280,8 +286,6 @@ class GGZChess:
 
     def onStart(self):
         print ('onStart',)
-        g = self.ui.application.addNetworkGame('Network Game')
-        
         # Create remote player
         if self.seatNum == 0:
             name = self.blackName
@@ -290,15 +294,15 @@ class GGZChess:
         self.remotePlayer = game.ChessPlayer(name)
         self.remotePlayer.onPlayerMoved = self.onPlayerMoved # FIXME: HACK HACK HACK!
 
-        p = g.addHumanPlayer('Human')
+        p = self.game.addHumanPlayer('Human')
         if self.seatNum == 0:
-            g.setWhite(p)
-            g.setBlack(self.remotePlayer)
+            self.game.setWhite(p)
+            self.game.setBlack(self.remotePlayer)
         else:
-            g.setWhite(self.remotePlayer)
-            g.setBlack(p)
+            self.game.setWhite(self.remotePlayer)
+            self.game.setBlack(p)
 
-        g.start()
+        self.game.start()
 
     def onPlayerMoved(self, player, move):
         #FIXME: HACK HACK HACK!
@@ -332,10 +336,12 @@ class GGZNetworkDialog(ui.NetworkFeedback):
             self.decoder.close()
         self.decoder = None
 
+        self.controller.clearError()       
         self.controller.setSensitive(False)
         self.controller.clearRooms()
         self.controller.clearPlayers()
         self.controller.clearTables()
+        self.controller.clearText()
         
         self.profile = profile
         if profile is None:
@@ -349,14 +355,29 @@ class GGZNetworkDialog(ui.NetworkFeedback):
         print 'Entering room %s' % room.id
         self.decoder._performAction(self.decoder.client.enterRoom, (room,))
         
+    def _addGame(self):
+        self.game = self.ui.application.addGame('Network Game')
+        if self.game is None:
+            # FIXME: Notify user game aborted
+            return False
+        return True
+        
     def startTable(self):
         """Called by ui.NetworkFeedback"""
-        if self.decoder.room.game is not None:
-            print 'Starting table'
-            self.decoder.client.startTable(self.decoder.room.game.id, 'glChess test game (do not join!)', self.profile.login)
-    
+        if self.decoder.room.game is None:
+            return
+        
+        if not self._addGame():
+            return
+        
+        print 'Starting table'
+        self.decoder.client.startTable(self.decoder.room.game.id, '', self.profile.login)
+
     def joinTable(self, table):
         """Called by ui.NetworkFeedback"""
+        if not self._addGame():
+            return
+        
         print 'Starting table %s' % table.id
         self.decoder.client.joinTable(table)
 
