@@ -70,11 +70,24 @@ public:
     virtual void drawCell (cairo_t * cr, gint x, gint y);
 
 private:
-  const static double border = 0.2;
+  double border;
   void drawInnerCorner (cairo_t * cr);
   void drawOuterCorner (cairo_t * cr);
   void drawHEdge (cairo_t * cr);
   void drawVEdge (cairo_t * cr);
 };
 
+class TangoBlock:public Renderer {
+public:
+  TangoBlock (cairo_surface_t * dst, cairo_surface_t * bg, Block ** src,
+	    int w, int h, int pxw, int pxh, gboolean grad);
+
+protected:
+  virtual void drawCell (cairo_t * cr, gint x, gint y);
+  gboolean usegrads;
+
+private:
+  void drawRoundedRectangle (cairo_t * cr, gdouble x, gdouble y, gdouble w, gdouble h, gdouble r); 
+};
+
 #endif // __renderer_h__
